library dynamic_middleware;

import 'package:redux/redux.dart';

/// [MiddlewareClass] used to add and remove other [Middleware] dynamically.
///
/// Executes other [Middleware<State>] as part of its own middleware [call] method.
class DynamicMiddleware<State> extends MiddlewareClass<State> {
  /// A [Set] of [Middleware<State>] objects with a [call] method.
  final Set<Middleware<State>> middlewares;

  /// Create a piece of middleware that adds a single piece of [middleware] as dynamic-middleware
  DynamicMiddleware._(this.middlewares);

  /// Create a piece of middleware to allow future add/remove of middleware.
  factory DynamicMiddleware.empty() =>
      new DynamicMiddleware._(new Set<Middleware<State>>());

  /// Create a piece of middleware that a single [middleware] as dynamic-middleware
  factory DynamicMiddleware.fromSingle(Middleware<State> middleware) =>
      new DynamicMiddleware._(new Set<Middleware<State>>.from([middleware]));

  /// Create a piece of middleware that adds all [middlewareList] as dynamic-middleware
  factory DynamicMiddleware.fromList(List<Middleware<State>> middlewareList) =>
      new DynamicMiddleware._(new Set<Middleware<State>>.from(middlewareList));

  @override
  void call(Store<State> store, Object action, NextDispatcher next) {
    if (middlewares.isEmpty) {
      next(action);
    } else {
      final copyWares = new Set.from(middlewares);
      var nextAction = action;
      for (var ware in copyWares) {
        if (ware == copyWares.last) {
          // last middleware called chains to regular next
          ware(store, nextAction, next);
        } else {
          // let [NextDispatcher] chain into for-loop iteration
          var shouldBreak = true;
          ware(store, action, (action) {
            nextAction = action;
            shouldBreak = false;
          });
          if (shouldBreak) {
            break;
          }
        }
      }
    }
  }

  /// Add a single piece of middleware dynamically.
  void add(Middleware<State> middleware) => middlewares.add(middleware);

  /// All a list of middleware dynamically.
  void addAll(List<Middleware<State>> middleware) =>
      middlewares.addAll(middleware);

  /// Remove a piece of middleware dynamically.
  void remove(Middleware<State> middleware) => middlewares.remove(middleware);

  /// Remove all middleware previously added.
  void clear() => middlewares.clear();
}
